#include "highlighter.h"
#include <QDebug>

Highlighter::Highlighter(const QString _filename, QObject *parent) :
    QSyntaxHighlighter(parent)
{
    xml_f=_filename;
    QFile file(xml_f);
    QStringList keywords;

    if (file.open(QIODevice::ReadOnly))
    {
        HighlightPrec hformat;
        QDomDocument document;
        document.setContent(&file);
        auto languages = document.elementsByTagName("language");
        for (int i = 0; i < languages.length(); ++i)
        {
            auto ext = languages.at(i).toElement();
            auto file_extension = QRegExp(ext.attribute("extentions"));
            HighlightLanguage data;

            QDomNodeList list = ext.elementsByTagName("keyword");
            int size = list.size();
            for (int i = 0; i < size; ++i)
                keywords.append(list.at(i).toElement().text());

            data.keywordFormat.setForeground(Qt::darkBlue);
            data.keywordFormat.setFontWeight(QFont::Bold);

            foreach (const QString& _pattern, keywords)
            {
                hformat.pattern = QRegExp("\\b"+_pattern+"\\b");
                hformat.format = data.keywordFormat;
                data.highlightPrecs.append(hformat);
            }
            keywords.clear();
            list = ext.elementsByTagName("command");
            size = list.size();
            data.preprocFormat.setForeground(Qt::darkBlue);

            for (int i = 0; i < size; ++i)
                keywords.append(list.at(i).toElement().text());

            foreach(const QString& _pattern, keywords)
            {
                hformat.pattern = QRegExp(_pattern);
                hformat.format = data.preprocFormat;
                data.highlightPrecs.append(hformat);
            }

            data.classFormat.setForeground(Qt::darkMagenta);
            data.classFormat.setFontWeight(QFont::Bold);
            hformat.pattern = QRegExp("\\bQ[A-Za-z]+\\b");
            hformat.format = data.classFormat;
            data.highlightPrecs.append(hformat);
            data.multiLineCommentFormat.setForeground(Qt::darkGreen);

            QDomNode beg_com = ext.elementsByTagName("multiline_comment").at(0);
            data.commentStExp=QRegExp(beg_com.toElement().attribute("begin"));
            data.commentEndExp=QRegExp(beg_com.toElement().attribute("end"));
            data.singleLineCommentFormat.setForeground(Qt::darkGreen);

            QDomNode single_com = ext.elementsByTagName("single_comment").at(0);
            hformat.pattern = QRegExp(single_com.toElement().attribute("expression"));
            hformat.format = data.singleLineCommentFormat;
            data.highlightPrecs.append(hformat);
            data.quotationFormat.setForeground(Qt::darkGreen);
            hformat.pattern = QRegExp("\".*\"");
            hformat.format = data.quotationFormat;
            data.highlightPrecs.append(hformat);
            data.functionFormat.setFontItalic(true);
            data.functionFormat.setForeground(Qt::blue);
            hformat.pattern = QRegExp("\\b[A-Za-z0-9_]+(?=\\()");

            hformat.format = data.functionFormat;
            data.highlightPrecs.append(hformat);
            languagesContainer.insert(file_extension, data);
        }
    }
}

void Highlighter::highlightBlock(const QString& text)
{
    auto &data = languagesContainer.value(current_extension);

    foreach (const HighlightPrec& rule_, data.highlightPrecs)
    {
        QRegExp expression(rule_.pattern);
        int index = expression.indexIn(text);
        while (index >= 0)
        {
            int len = expression.matchedLength();
            setFormat(index, len, rule_.format);
            index = expression.indexIn(text, index+len);
        }
    }
    setCurrentBlockState(0);
    int begin = 0;
    if (previousBlockState() != 1)
        begin = data.commentStExp.indexIn(text);

    while (begin >= 1)
    {
        int end = data.commentEndExp.indexIn(text);
        int c_len;
        if (end == -1)
        {
            setCurrentBlockState(1);
            c_len = text.length() - begin + data.commentEndExp.matchedLength();
        }
        else
            c_len = end - begin + data.commentEndExp.matchedLength();

        setFormat(begin, c_len, data.multiLineCommentFormat);
        begin = data.commentStExp.indexIn(text, begin + c_len);
    }
}

bool Highlighter::setExtension(const QString &_filename)
{
    for (int i = 0; i < languagesContainer.size(); ++i)
    {
        if (languagesContainer.keys().at(i).indexIn(_filename) != -1)
        {
            current_extension = languagesContainer.keys().at(i);
            return true;
        }
    }
    return false;
}
